package handwriting

import (
	"bytes"
	context "context"
	fmt "fmt"
	"net"

	neuralnetwork "../../neuralnetwork/neuralnetwork"
	"gonum.org/v1/gonum/mat"
	grpc "google.golang.org/grpc"
)

type HandwritingService struct {
	nn *neuralnetwork.NeuralNetwork
}

func NewHandwritingService() (hws *HandwritingService) {
	hws = &HandwritingService{}
	hws.nn, _ = neuralnetwork.NewNeuralNetwork([]int{784, 24, 24, 10}, neuralnetwork.NewRPropInitializer(neuralnetwork.RPropConfig{
		NuPlus:   1.2,
		NuMinus:  0.5,
		DeltaMax: 50.0,
		DeltaMin: 0.000001,
	}))
	return
}

func (hws *HandwritingService) Recognize(ctx context.Context, matrix *Matrix) (*Result, error) {
	fmt.Printf("Recognize %v size: %v\n", len(matrix.Data), hws.nn.Sizes[0])
	dense := mat.NewDense(hws.nn.Sizes[0], 1, matrix.Data)
	index, _ := hws.nn.Predict(dense)
	fmt.Printf("Recognition result %v\n", index)
	return &Result{ResultCharacter: uint32(index)}, nil
}

func (hws *HandwritingService) SetNeuralNetworkData(ctx context.Context, nnRaw *NeuralNetworkRaw) (*None, error) {
	fmt.Println("SetNeuralNetworkData")
	r := bytes.NewReader(nnRaw.Data)
	hws.nn.LoadState(r)
	return &None{}, nil
}

func (hws *HandwritingService) Run() {
	grpcServer := grpc.NewServer()
	RegisterHandwritingServer(grpcServer, hws)
	lis, err := net.Listen("tcp", "localhost:65001")
	if err != nil {
		fmt.Printf("Failed to listen: %v\n", err)
	}

	fmt.Printf("Listen localhost:65001\n")
	if err := grpcServer.Serve(lis); err != nil {
		fmt.Printf("Failed to serve: %v\n", err)
	}
}
