package mutagen

import (
	"math/rand"
	"time"

	neuralnetwork "../../neuralnetwork"
)

type DummyMutagen struct {
	chance        float64
	mutationCount int
}

func NewDummyMutagen(chance float64, mutationCount int) (dm *DummyMutagen) {
	dm = &DummyMutagen{
		chance:        chance,
		mutationCount: mutationCount,
	}
	return
}

func (dm *DummyMutagen) Mutate(network *neuralnetwork.NeuralNetwork) {
	rand.Seed(time.Now().UnixNano())
	for l := 1; l < network.LayerCount; l++ {
		randomized := rand.Float64()
		if randomized < dm.chance {
			r, c := network.Weights[l].Dims()
			for o := 0; o < dm.mutationCount; o++ {
				mutationRow := int(rand.Uint32()) % r
				mutationColumn := int(rand.Uint32()) % c
				weight := rand.NormFloat64()
				bias := rand.NormFloat64()
				network.Weights[l].Set(mutationRow, mutationColumn, weight)
				network.Biases[l].Set(mutationRow, 0, bias)
			}
		}
	}
}
